@@ -11,6 +11,7 @@ import glob
 import math
 import copy
 import random
+import inspect
 
 import muteria.common.mix as common_mix
 import muteria.common.fs as common_fs
@@ -331,6 +332,210 @@ class Executor(object):
         return self.repo_mgr
     #~ def get_repo_manager()
 
+    def custom_execution(self):
+        """ Interact with the user and ask what to do.
+            For execution of tests with custom executable for example
+        """
+        # XXX: check that the normal execution was finished (from checkpointer)
+        self.checkpointer = \
+                    common_fs.CheckpointState(*self._get_checkpoint_files())
+        ERROR_HANDLER.assert_true(self.checkpointer.is_finished(), \
+                        "custom execution must be ran when run is finished", \
+                                                                    __file__)
+
+        tests_key = 'tests'
+        criteria_tests_key = 'criteria_tests'
+        mode = None
+        mode = input ("> Chose what to execute ({} or {}): ".format(\
+                                                tests_key, criteria_tests_key))
+        ERROR_HANDLER.assert_true(mode in [tests_key, criteria_tests_key], \
+                                "Invalid mode entered: " + mode, __file__)
+
+        custom_out = input("> Input the {} custom output dir: ".format(\
+                                "non existing" if mode == tests_key else ""))
+        custom_out = os.path.abspath(custom_out)
+        ERROR_HANDLER.assert_true(os.path.isdir(os.path.dirname(custom_out)), \
+                            "parent of custom_out is not existing (" \
+                                +os.path.dirname(custom_out)+')', __file__)
+        if mode == tests_key:
+            ERROR_HANDLER.assert_true(not os.path.isdir(custom_out), \
+                        "custom_out is existing ("+custom_out+')', __file__)
+
+            ## read exe
+            exe_map_str = input("> Input existing executable file map to use"
+                        " ({<path in repo>: <used exe path>}): ")
+            exe_map = common_fs.json.loads(exe_map_str)
+            ERROR_HANDLER.assert_true(type(exe_map) == dict, \
+                                                "invalid path map", __file__)
+            for kp, rp in exe_map.items():
+                ERROR_HANDLER.assert_true(os.path.isfile(rp), \
+                            "executable is missing ({}: {})".format(kp, rp), \
+                                                                    __file__)
+            ## read test list
+            testlist_file = input("> Input existing test list file to use: ")
+            ERROR_HANDLER.assert_true(os.path.isfile(testlist_file), \
+                    "test file list not existing ({})".format(testlist_file),\
+                                                                    __file__)
+            with open(testlist_file) as f:
+                testlist = [t.strip() for t in f]
+
+            # XXX execution
+            ## Create meta test tool
+            try:
+                meta_testcase_tool = self.meta_testcase_tool
+            except AttributeError:
+                meta_testcase_tool = self._create_meta_test_tool(self.config, \
+                                                            self.head_explorer)
+                meta_testcase_tool.check_tools_installed()
+            ## create test prioritization tool
+            try:
+                meta_testexec_optimization_tool = \
+                                        self.meta_testexec_optimization_tool
+            except AttributeError:
+                meta_testexec_optimization_tool = \
+                        self._create_meta_testexec_optimization(self.config)
+
+            ## Create outdir
+            os.mkdir(custom_out)
+
+            ## run the tests
+
+            matrix_file_key = outdir_struct.TEST_PASS_FAIL_MATRIX
+            matrix_file = os.path.join(custom_out, matrix_file_key)
+            execoutput_file_key = \
+                                outdir_struct.PROGRAM_TESTEXECUTION_OUTPUT
+            execoutput_file = None
+            if self.config.GET_PASSFAIL_OUTPUT_SUMMARY.get_val():
+                execoutput_file = os.path.join(custom_out, execoutput_file_key)
+
+            meta_testcase_tool.runtests(meta_testcases=testlist, \
+                        exe_path_map=exe_map,
+                        stop_on_failure=\
+                                self.config.STOP_TESTS_EXECUTION_ON_FAILURE\
+                                                                .get_val(), \
+                        recalculate_execution_times=False, \
+                        fault_test_execution_matrix_file=matrix_file, \
+                        fault_test_execution_execoutput_file=execoutput_file, \
+                        test_prioritization_module=\
+                                        meta_testexec_optimization_tool, \
+                        finish_destroy_checkpointer=True)
+        elif mode == criteria_tests_key:
+            print ("# This mode is just needed to reexecute tests with"
+                                        "some test criteria test objectives")
+            
+            ## read test criterion
+            #all_criteria_str = [c.get_str() for c in criteria_pkg.TestCriteria]
+            enabled_criteria = self.config.ENABLED_CRITERIA.get_val()
+            criterion = input("> Input the test criterion to use "
+                                "(choose from {}): ".format(enabled_criteria))
+            if not isinstance(criterion, criteria_pkg.TestCriteria):
+                ERROR_HANDLER.assert_true(\
+                        criteria_pkg.TestCriteria.has_element_named(\
+                            criterion), \
+                        "invalid test criterion ({}). choose from: {}".format(\
+                            criterion, enabled_criteria),\
+                        __file__)
+                criterion = criteria_pkg.TestCriteria[criterion] 
+            ERROR_HANDLER.assert_true(criterion in enabled_criteria,
+                        "The chosen criterion is not in the config Enabled"
+                        "criteria", __file__)
+
+            ## read test list
+            to_test_map_file = input("> Input existing test-objective to test"
+                                                        " map file to use: ")
+            ERROR_HANDLER.assert_true(os.path.isfile(to_test_map_file), \
+                    "test-objective to test map file list not existing ({})".\
+                                        format(to_test_map_file), __file__)
+            to_test_map = common_fs.loadJSON(to_test_map_file)
+
+            # XXX execution
+            ## Create meta test tool
+            try:
+                meta_testcase_tool = self.meta_testcase_tool
+            except AttributeError:
+                meta_testcase_tool = self._create_meta_test_tool(self.config, \
+                                                            self.head_explorer)
+                meta_testcase_tool.check_tools_installed()
+
+            # Meta criteria
+            try:
+                meta_criteria_tool = self.meta_criteria_tool 
+            except AttributeError:
+                meta_criteria_tool = self._create_meta_criteria_tool(\
+                                            self.config, meta_testcase_tool)
+                meta_criteria_tool.check_tools_installed()
+
+            # Criteria optimization
+            #try:
+            #    meta_criteriaexec_optimization_tools = \
+            #                        self.meta_criteriaexec_optimization_tools
+            #except AttributeError:
+            #meta_criteriaexec_optimization_tools = \
+            #        self._create_meta_criteriaexec_optimization(self.config)
+            ### WE DO NOT USE CONF OPTIMIZER 
+            meta_criteriaexec_optimization_tools = {criterion:
+                    crit_opt_module.CriteriaOptimizers\
+                            .OPTIMIZED_FROM_DICT.get_optimizer()(self.config, \
+                                        self.head_explorer, criterion, \
+                                                        dictobj=to_test_map)
+            }
+
+            ## run 
+            output_file = None
+            if criterion in self.config.CRITERIA_WITH_OUTPUT_SUMMARY.get_val():
+                output_file = os.path.join(custom_out, \
+                            outdir_struct.CRITERIA_EXECUTION_OUTPUT[criterion])
+            matrix_file = os.path.join(custom_out, \
+                                outdir_struct.CRITERIA_MATRIX[criterion])
+            
+            all_test_list = set()
+            for _, test_list in to_test_map.items():
+                all_test_list |= set(test_list)
+            all_test_list = list(all_test_list)
+            if len(all_test_list) != 0:
+                ## Create outdir
+                if not os.path.isdir(custom_out):
+                    os.mkdir(custom_out)
+
+                meta_criteria_tool.runtests_criteria_coverage( \
+                            testcases=all_test_list, \
+                            criterion_to_matrix={criterion: matrix_file}, \
+                            criterion_to_executionoutput=\
+                                                {criterion: output_file}, \
+                            criteria_element_list_by_criteria=\
+                                            {criterion: list(to_test_map)}, \
+                            cover_criteria_elements_once=self.config.\
+                                    COVER_CRITERIA_ELEMENTS_ONCE.get_val(),\
+                            prioritization_module_by_criteria=\
+                                    meta_criteriaexec_optimization_tools,\
+                            finish_destroy_checkpointer=True)
+                
+                os.remove(matrix_file)
+
+                # XXX The matrix is meaningless here
+                # Update matrix if needed to have output diff or such
+                #if criterion in set(self.config\
+                #        .CRITERIA_REQUIRING_OUTDIFF_WITH_PROGRAM.get_val()):
+                #    pf_matrix_file = self.head_explorer.get_file_pathname(\
+                #                    outdir_struct.TEST_PASS_FAIL_MATRIX)
+                #    ERROR_HANDLER.assert_true(os.path.isfile(pf_matrix_file),\
+                #                        "pass-fail matrix missing.", __file__)
+                #    if self.config.GET_PASSFAIL_OUTPUT_SUMMARY.get_val():
+                #        pf_execoutput_file = \
+                #                        self.head_explorer.get_file_pathname(\
+                #                outdir_struct.PROGRAM_TESTEXECUTION_OUTPUT)
+                #        ERROR_HANDLER.assert_true(\
+                #                        os.path.isfile(pf_execoutput_file),\
+                #                        "exec output file missing.", __file__)
+                #    else:
+                #        pf_execoutput_file = None
+                #    DriversUtils.update_matrix_to_cover_when_difference(\
+                #                            matrix_file,  output_file, \
+                #                            pf_matrix_file, pf_execoutput_file)
+        else:
+            ERROR_HANDLER.error_exit("invalid mode and bug", __file__)
+    #~ def custom_execution()
+
     def _execute_task(self, task):
         """ TODO: 
                 1. implement the todos here
@@ -421,7 +626,25 @@ class Executor(object):
                 if toolalias in candidate_aliases:
                     selected_tests.append(meta_test)
 
-            # TODO: use the actual selector. For now just use all tests
+            # XXX use the actual selector. For now just use all tests
+            sel_tech = self.config.TESTCASES_SELECTION.get_val()
+            # selection method
+            if inspect.isfunction(sel_tech):
+                ERROR_HANDLER.assert_true(\
+                                    len(inspect.getargspec().args) == 2, \
+                                    "Test selection function must take 2 args "
+                                    "(testlist, maxselcount)", __file__)
+                
+            else:
+                if sel_tech is not None and sel_tech != 'DummyRandom':
+                    ERROR_HANDLER.error_exit(\
+                        'only random tests selection supported now!', \
+                                                                __file__)
+                # make DummyRandom selection
+                sel_tech = random.sample
+
+            # make selection
+            selected_tests = sel_tech(selected_tests, len(selected_tests))
 
             # Check for flakiness
             logging.debug("# Checking for tests flakiness ...")
@@ -553,13 +776,8 @@ class Executor(object):
                 if info_obj is None:
                     continue
                 all_to = info_obj.get_elements_list()
-                # make selection
-                if sel_tech != 'DummyRandom':
-                    ERROR_HANDLER.error_exit(\
-                        'only random TO selection supported now!', __file__)
-
-                # make DummyRandom selection
-                sel_count =  self.config\
+                
+                sel_count = self.config\
                             .MAX_CRITERIA_ELEM_SELECTION_NUM_PERCENT.get_val()
                 if type(sel_count) == str:
                     if sel_count.endswith('%'):
@@ -581,11 +799,25 @@ class Executor(object):
                                                                     __file__)
                 ERROR_HANDLER.assert_true(sel_count > 0, \
                                 "selection number must be positive", __file__)
-                if sel_count >= len(all_to):
-                    selected_TO[crit.get_str()] = all_to
+
+                # selection method
+                if inspect.isfunction(sel_tech):
+                    ERROR_HANDLER.assert_true(\
+                                        len(inspect.getargspec().args) == 2, \
+                                        "Selection function must take 2 args "
+                                        "(testobjectivelist, maxselcount)", \
+                                        __file__)
+                    
                 else:
-                    selected_TO[crit.get_str()] = random.sample(all_to, \
-                                                                    sel_count)
+                    if sel_tech != 'DummyRandom':
+                        ERROR_HANDLER.error_exit(\
+                            'only random TO selection supported now!', \
+                                                                    __file__)
+                    # make DummyRandom selection
+                    sel_tech = random.sample
+
+                # make selection
+                selected_TO[crit.get_str()] = sel_tech(all_to, sel_count)
             
             # write down selection
             common_fs.dumpJSON(selected_TO, out_file)
@@ -854,7 +1086,8 @@ class Executor(object):
                         code_builds_factory=self.cb_factory,
                         test_tool_config_list=\
                                     config.TESTCASE_TOOLS_CONFIGS.get_val(),\
-                        head_explorer=head_explorer)
+                        head_explorer=head_explorer, \
+                        hash_outlog=config.HASH_OUTLOG.get_val())
         return meta_test_tool
     #~ def _create_meta_test_tool()
 
