@@ -663,9 +663,11 @@ class BaseCriteriaTool(abc.ABC):
         try:
             for root_, dirs_, files_ in os.walk(dirpath):
                 for sub_d in dirs_:
-                    os.chmod(os.path.join(root_, sub_d), 0o777)
+                    if os.path.isdir(os.path.join(root_, sub_d)):
+                        os.chmod(os.path.join(root_, sub_d), 0o777)
                 for f_ in files_:
-                    os.chmod(os.path.join(root_, f_), 0o777)
+                    if os.path.isfile(os.path.join(root_, f_)):
+                        os.chmod(os.path.join(root_, f_), 0o777)
         except PermissionError:
             ret,_,_ = DriversUtils.execute_and_get_retcode_out_err('sudo', \
                                         ['chmod 777 -R {}'.format(dirpath)])
