@@ -99,7 +99,8 @@ class MetaTestcaseTool(object):
     #~ def get_toolnames_by_types_by_language()
 
     def __init__(self, language, tests_working_dir, code_builds_factory,
-                                test_tool_config_list, head_explorer):
+                                test_tool_config_list, head_explorer, 
+                                hash_outlog=True):
 
         """ Initialize a meta testcase tool object.
         :type language:
@@ -128,6 +129,7 @@ class MetaTestcaseTool(object):
         self.code_builds_factory = code_builds_factory
         self.test_tool_config_list = test_tool_config_list
         self.head_explorer = head_explorer
+        self.hash_outlog = hash_outlog
 
         # Verify Direct Arguments Variables
         ERROR_HANDLER.assert_true(self.tests_working_dir is not None, \
@@ -261,7 +263,7 @@ class MetaTestcaseTool(object):
                                         use_recorded_timeout_times=None, \
                                         recalculate_execution_times=False, \
                                         with_output_summary=True, \
-                                        hash_outlog=True):
+                                        hash_outlog=None):
         '''
         Execute a test case with the given executable and 
         say whether it failed
@@ -271,6 +273,8 @@ class MetaTestcaseTool(object):
                         the executable to execute with the test
         :param env_vars: dict of environment variables to set before
                         executing the test ({<variable>: <value>})
+        :type hash_outlog: bool
+        :hash_outlog: decide whether to hash the outlog or not
         :returns: pair of:
                 - boolean failed verdict of the test 
                         (True if failed, False otherwise)
@@ -280,6 +284,9 @@ class MetaTestcaseTool(object):
         if exe_path_map is None:
             exe_path_map = self._get_default_exe_path_map()
 
+        if hash_outlog is None:
+            hash_outlog = self.hash_outlog
+
         # Find which test tool's the testcase is, then execute
         ttoolalias, testcase = DriversUtils.reverse_meta_element(meta_testcase)
         ERROR_HANDLER.assert_true( \
@@ -305,7 +312,7 @@ class MetaTestcaseTool(object):
                         fault_test_execution_matrix_file=None, \
                         fault_test_execution_execoutput_file=None, \
                         with_output_summary=True, \
-                        hash_outlog=True, \
+                        hash_outlog=None, \
                         test_prioritization_module=None, \
                         parallel_test_count=1, \
                         parallel_test_scheduler=None, \
@@ -327,6 +334,8 @@ class MetaTestcaseTool(object):
         :param fault_test_execution_execoutput_file: Optional output log file 
                         to store the tests' execution actual output (hashed)
         :param with_output_summary: decide whether to return outlog hash 
+        :type hash_outlog: bool
+        :hash_outlog: decide whether to hash the outlog or not
         :param test_prioritization_module: Specify the test prioritization
                         module. 
                         (TODO: Implement support)
@@ -372,6 +381,9 @@ class MetaTestcaseTool(object):
         if exe_path_map is None:
             exe_path_map = self._get_default_exe_path_map()
 
+        if hash_outlog is None:
+            hash_outlog = self.hash_outlog
+
         ERROR_HANDLER.assert_true(parallel_test_count is None \
                                         or parallel_test_count >= 1, \
                                 "invalid parallel tests count ({})".format(\
@@ -429,26 +441,60 @@ class MetaTestcaseTool(object):
 
         shared_loc = multiprocessing.RLock()
 
-        next_parallel_count = parallel_test_count
+        parallel_test_count_by_tool = {ta: 1 for ta in candidate_aliases}
+
+        # tool with parallel test exec
+        # TODO: find way to pass parallel count here
+        if parallel_test_count is None:
+            #parallel_test_count = min(10, multiprocessing.cpu_count())
+            parallel_test_count = min(20, 2*multiprocessing.cpu_count())
+        
+        # use parallel
+        sub_parallel_count = 0 if parallel_test_count is None else \
+                        parallel_test_count - len(parallel_test_count_by_tool)
+        if sub_parallel_count > 0:
+            para_tools = [tt for tt in candidate_aliases if \
+                                self.testcases_configured_tools[tt]\
+                                [self.TOOL_OBJ_KEY].can_run_tests_in_parallel()
+                        ]
+            para_tools.sort(reverse=True, \
+                                    key=lambda x: len(testcases_by_tool[x]))
+            para_tools_n_tests = sum(\
+                            [len(testcases_by_tool[tt]) for tt in para_tools])
+            
+            used = 0
+            for tt in para_tools:
+                quota = int(len(testcases_by_tool[tt]) * sub_parallel_count / \
+                                                            para_tools_n_tests)
+                parallel_test_count_by_tool[tt] += quota
+                used += quota
+            for tt in para_tools:
+                if used == sub_parallel_count:
+                    break
+                parallel_test_count_by_tool[tt] += 1
+
 
         def tool_parallel_test_exec(ttoolalias):
             # Actual execution
-            found_a_failure=False
+            found_a_failure = False
+            # Whether the execution was unsuccessful
+            test_error = False
             ttool = \
                 self.testcases_configured_tools[ttoolalias][self.TOOL_OBJ_KEY]
             test_failed_verdicts, test_execoutput = ttool.runtests( \
-                                            testcases_by_tool[ttoolalias], \
-                                            exe_path_map, env_vars, \
-                                            stop_on_failure, \
-                                            per_test_timeout=per_test_timeout,
-                                            use_recorded_timeout_times=\
-                                                use_recorded_timeout_times, \
-                                            recalculate_execution_times=\
-                                                recalculate_execution_times, \
-                                            with_output_summary=\
-                                                        with_output_summary, \
-                                            hash_outlog=hash_outlog, \
-                                            parallel_count=next_parallel_count)
+                                testcases_by_tool[ttoolalias], \
+                                exe_path_map, env_vars, \
+                                stop_on_failure, \
+                                per_test_timeout=per_test_timeout,
+                                use_recorded_timeout_times=\
+                                    use_recorded_timeout_times, \
+                                recalculate_execution_times=\
+                                    recalculate_execution_times, \
+                                with_output_summary=\
+                                            with_output_summary, \
+                                hash_outlog=hash_outlog, \
+                                parallel_count=\
+                                    parallel_test_count_by_tool[ttoolalias])
             with shared_loc:
                 for testcase in test_failed_verdicts:
                     meta_testcase =  DriversUtils.make_meta_element(\
@@ -459,35 +505,35 @@ class MetaTestcaseTool(object):
                                                     test_execoutput[testcase]
                     if not found_a_failure \
                                 and test_failed_verdicts[testcase] == \
-                                common_mix.GlobalConstants.COMMAND_UNCERTAIN:
+                                common_mix.GlobalConstants.FAIL_TEST_VERDICT:
                         found_a_failure = True
+                    if not test_error \
+                                and test_failed_verdicts[testcase] == \
+                            common_mix.GlobalConstants.TEST_EXECUTION_ERROR:
+                        test_error = True
 
                 # @Checkpoint: Chekpointing
                 checkpoint_handler.do_checkpoint(func_name=cp_func_name, \
                                 taskid=cp_task_id, \
                                 tool=ttoolalias, \
                                 opt_payload=meta_test_failedverdicts_outlog)
-            return found_a_failure
+            return found_a_failure, test_error
         #~ def tool_parallel_test_exec()
 
         # minimum number of tests for parallelism
         ptest_tresh = 5
 
         if len(candidate_aliases) > 1 and len(meta_testcases) >= ptest_tresh \
-                and (parallel_test_count is None or parallel_test_count > 1):
-            if parallel_test_count is None:
-                paralle_count = min(len(candidate_aliases), \
-                                                multiprocessing.cpu_count())
-            else:
-                paralle_count = min(len(candidate_aliases), \
-                                                        parallel_test_count)
-                #next_parallel_count = 1
+                                        and parallel_test_count is not None \
+                                        and parallel_test_count > 1:
+            paralle_count = min(len(candidate_aliases), parallel_test_count)
             joblib.Parallel(n_jobs=paralle_count, require='sharedmem')\
                     (joblib.delayed(tool_parallel_test_exec)(ttoolalias) \
                         for ttoolalias in candidate_aliases)
         else:
             for tpos, ttoolalias in enumerate(candidate_aliases):
-                found_a_failure = tool_parallel_test_exec(ttoolalias)
+                found_a_failure, test_error = \
+                                        tool_parallel_test_exec(ttoolalias)
                 if stop_on_failure and found_a_failure:
                     # @Checkpoint: Chekpointing for remaining tools
                     for rem_tool in list(testcases_by_tool.keys())[tpos+1:]:
@@ -511,7 +557,14 @@ class MetaTestcaseTool(object):
                     
         ERROR_HANDLER.assert_true(len(meta_test_failedverdicts_outlog[0]) == \
                                                         len(meta_testcases), \
-                            "Not all tests have a verdict reported", __file__)
+                    "mismatch between number of tests and reported verdicts:"
+                    + " Tests without verdict are {};".format(\
+                               set(meta_testcases) - \
+                                    set(meta_test_failedverdicts_outlog[0])) \
+                    + " Test not in testlist are {}.".format(\
+                               set(meta_test_failedverdicts_outlog[0]) - \
+                                    set(meta_testcases)), \
+                                                                     __file__)
 
         if fault_test_execution_matrix_file is not None:
             # Load or Create the matrix 
@@ -651,7 +704,8 @@ class MetaTestcaseTool(object):
         # @Checkpoint: create a checkpoint handler
         cp_func_name = "generate_tests"
         if test_tool_type_list is not None:
-            for test_tool_type in sorted(test_tool_type_list):
+            for test_tool_type in sorted(test_tool_type_list, \
+													key=lambda x: x.get_str()):
                 cp_func_name += ":" + test_tool_type.get_str()
         cp_task_id = 1
         checkpoint_handler = CheckPointHandler(\
